@@ -6,15 +6,22 @@ import tempfile
 import xml.etree.ElementTree as ET
 import zipfile
 from errno import ENOENT
-from shutil import rmtree
 from tempfile import mkdtemp
 from typing import List, Optional, Dict, Any, IO, Tuple
 from xml.etree.ElementTree import Element
 
+from pandas.io.parsers import TextFileReader
+
 import dwca.vendor
 from dwca.descriptors import ArchiveDescriptor, DataFileDescriptor, shorten_term
-from dwca.exceptions import RowNotFound, InvalidArchive, InvalidSimpleArchive, NotADataFile
+from dwca.exceptions import (
+    RowNotFound,
+    InvalidArchive,
+    InvalidSimpleArchive,
+    NotADataFile,
+)
 from dwca.files import CSVDataFile
+from dwca.helpers import remove_tree
 from dwca.rows import CoreRow
 
 
@@ -70,7 +77,7 @@ class DwCAReader(object):
 
     default_metadata_filename = "EML.xml"
     default_metafile_name = "meta.xml"
-    source_metadata_directory = 'dataset'
+    source_metadata_directory = "dataset"
 
     def __enter__(self):
         return self
@@ -78,7 +85,13 @@ class DwCAReader(object):
     def __exit__(self, exc_type, exc_value, exc_traceback):
         self.close()
 
-    def __init__(self, path: str, extensions_to_ignore: List[str] = None, tmp_dir: str = None) -> None:
+    def __init__(
+        self,
+        path: str,
+        extensions_to_ignore: List[str] = None,
+        tmp_dir: str = None,
+        skip_metadata: bool = False,
+    ) -> None:
         """Open the Darwin Core Archive."""
         if extensions_to_ignore is None:
             extensions_to_ignore = []
@@ -90,7 +103,9 @@ class DwCAReader(object):
         #: The path to the Darwin Core Archive file, as passed to the constructor.
         self.archive_path = path  # type: str
 
-        if os.path.isdir(self.archive_path):  # Archive is a (directly readable) directory
+        if os.path.isdir(
+            self.archive_path
+        ):  # Archive is a (directly readable) directory
             self._working_directory_path = self.archive_path
             self._directory_to_clean = None  # type: Optional[str]
         else:  # Archive is zipped/tgzipped, we have to extract it first.
@@ -99,16 +114,21 @@ class DwCAReader(object):
         #: An :class:`descriptors.ArchiveDescriptor` instance giving access to the archive
         #: descriptor/metafile (``meta.xml``)
         self.descriptor = None  # type: Optional[ArchiveDescriptor]
+        self._metafile_handle = None
         try:
-            self.descriptor = ArchiveDescriptor(self.open_included_file(self.default_metafile_name).read(),
-                                                files_to_ignore=extensions_to_ignore)
+            self._metafile_handle = self.open_included_file(self.default_metafile_name)
+            self.descriptor = ArchiveDescriptor(
+                self._metafile_handle.read(), files_to_ignore=extensions_to_ignore
+            )
         except IOError as exc:
             if exc.errno == ENOENT:
                 pass
 
         #: A :class:`xml.etree.ElementTree.Element` instance containing the (scientific) metadata
-        #: of the archive, or `None` if the archive has no metadata.
-        self.metadata = self._parse_metadata_file()  # type: Optional[Element]
+        #: of the archive, or `None` if the archive has no metadata or if the `skip_metadata` parameter is True.
+        self.metadata = None  # type: Optional[Element]
+        if not skip_metadata:
+            self.metadata = self._parse_metadata_file()  # type: Optional[Element]
 
         #: If the archive contains source-level metadata (typically, GBIF downloads), this is a dict such as::
         #:
@@ -118,23 +138,33 @@ class DwCAReader(object):
         #: See :doc:`gbif_results` for more details.
         self.source_metadata = self._get_source_metadata()  # type: Dict[str, Element]
 
-        if self.descriptor:  # We have an Archive descriptor that we can use to access data files.
+        if (
+            self.descriptor
+        ):  # We have an Archive descriptor that we can use to access data files.
             #: An instance of :class:`dwca.files.CSVDataFile` for the core data file.
-            self.core_file = CSVDataFile(self._working_directory_path, self.descriptor.core)  # type: CSVDataFile
+            self.core_file = CSVDataFile(
+                self._working_directory_path, self.descriptor.core
+            )  # type: CSVDataFile
 
             #: A list of :class:`dwca.files.CSVDataFile`, one entry for each extension data file , sorted by order of
             #: appearance in the Metafile (or an empty list if the archive doesn't use extensions).
-            self.extension_files = [CSVDataFile(work_directory=self._working_directory_path,
-                                                file_descriptor=d)
-                                    for d in self.descriptor.extensions]  # type: List[CSVDataFile]
+            self.extension_files = [
+                CSVDataFile(
+                    work_directory=self._working_directory_path, file_descriptor=d
+                )
+                for d in self.descriptor.extensions
+            ]  # type: List[CSVDataFile]
         else:  # Archive without descriptor, we'll have to find and inspect the data file
             try:
                 datafile_name = self._is_valid_simple_archive()
                 descriptor = DataFileDescriptor.make_from_file(
-                    os.path.join(self._working_directory_path, datafile_name))
+                    os.path.join(self._working_directory_path, datafile_name)
+                )
 
-                self.core_file = CSVDataFile(work_directory=self._working_directory_path,
-                                             file_descriptor=descriptor)
+                self.core_file = CSVDataFile(
+                    work_directory=self._working_directory_path,
+                    file_descriptor=descriptor,
+                )
                 self.extension_files = []
             except InvalidSimpleArchive:
                 msg = "No Metafile was found, but the archive contains multiple files/directories."
@@ -142,14 +172,17 @@ class DwCAReader(object):
 
     def _get_source_metadata(self) -> Dict[str, Element]:
         source_metadata = {}  # type: Dict[str, Element]
-        source_metadata_dir = os.path.join(self._working_directory_path, self.source_metadata_directory)
+        source_metadata_dir = os.path.join(
+            self._working_directory_path, self.source_metadata_directory
+        )
 
         if os.path.isdir(source_metadata_dir):
             for f in os.listdir(source_metadata_dir):
                 if os.path.isfile(os.path.join(source_metadata_dir, f)):
                     dataset_key = os.path.splitext(f)[0]
                     source_metadata[dataset_key] = self._parse_xml_included_file(
-                        os.path.join(self.source_metadata_directory, f))
+                        os.path.join(self.source_metadata_directory, f)
+                    )
 
         return source_metadata
 
@@ -184,29 +217,45 @@ class DwCAReader(object):
         .. note::
 
             Default values of Darwin Core Archive are supported: A column will be added to the DataFrame if a term has
-            a default value in the Metafile (but no corresponding column in the CSV Data File).
+            a default value in the Metafile (but no corresponding column in the CSV Data File). This is unfortunately
+            not supported in case the value returned by `pandas.read_csv()` is a `TextFileReader` (e.g. when using
+            `chunksize` or `iterator=True`).
         """
-        datafile_descriptor = self.get_descriptor_for(relative_path)  # type: DataFileDescriptor
+        datafile_descriptor = self.get_descriptor_for(
+            relative_path
+        )  # type: DataFileDescriptor
 
         if not dwca.vendor._has_pandas:
             raise ImportError("Pandas is missing.")
 
         from pandas import read_csv
 
-        kwargs['delimiter'] = datafile_descriptor.fields_terminated_by
-        kwargs['skiprows'] = datafile_descriptor.lines_to_ignore
-        kwargs['header'] = None
-        kwargs['names'] = datafile_descriptor.short_headers
+        kwargs["delimiter"] = datafile_descriptor.fields_terminated_by
+        kwargs["skiprows"] = datafile_descriptor.lines_to_ignore
+        kwargs["header"] = None
+        kwargs["names"] = datafile_descriptor.short_headers
 
-        df = read_csv(self.absolute_temporary_path(relative_path), **kwargs)
+        df_or_textreader = read_csv(
+            self.absolute_temporary_path(relative_path), **kwargs
+        )
 
         # Add a column for default values, if present in the file descriptor
         for field in datafile_descriptor.fields:
-            field_default_value = field['default']
+            field_default_value = field["default"]
             if field_default_value is not None:
-                df[shorten_term(field['term'])] = field_default_value
+                if isinstance(df_or_textreader, TextFileReader):
+                    # I don't see how to assign default values to a TextFileReader, so
+                    # this is currently unsupported
+                    raise ValueError(
+                        "Pandas read_csv() was called with a chunksize or iterator=True, "
+                        "and therefore returns a TextFileReader instead of a DataFrame "
+                        "which is not supported in combination with default values of "
+                        "the archive."
+                    )
 
-        return df
+                df_or_textreader[shorten_term(field["term"])] = field_default_value
+
+        return df_or_textreader
 
     def orphaned_extension_rows(self) -> Dict[str, Dict[str, List[int]]]:
         """Return a dict of the orphaned extension rows.
@@ -233,7 +282,9 @@ class DwCAReader(object):
             ids = temp_ids.keys()
 
             for extension in self.extension_files:
-                coreid_index = dict((k, v.tolist()) for k, v in extension.coreid_index.items())
+                coreid_index = dict(
+                    (k, v.tolist()) for k, v in extension.coreid_index.items()
+                )
                 for id in ids:
                     coreid_index.pop(id, None)
                 indexes[extension.file_descriptor.file_location] = coreid_index
@@ -295,7 +346,7 @@ class DwCAReader(object):
             - The position is often an appropriate way to unambiguously identify a core row in a DwCA.
 
         """
-        for (i, row) in enumerate(self):
+        for i, row in enumerate(self):
             if i == position:
                 return row
 
@@ -324,7 +375,9 @@ class DwCAReader(object):
             File existence is not tested.
 
         """
-        return os.path.abspath(os.path.join(self._working_directory_path, relative_path))
+        return os.path.abspath(
+            os.path.join(self._working_directory_path, relative_path)
+        )
 
     def get_descriptor_for(self, relative_path: str) -> DataFileDescriptor:
         """Return a descriptor for the data file located at relative_path.
@@ -349,7 +402,6 @@ class DwCAReader(object):
         raise NotADataFile("{fn} is not a data file".format(fn=relative_path))
 
     def _is_valid_simple_archive(self) -> str:
-
         # If the working dir appear to contains a valid simple darwin core archive
         # (one single data file + possibly some metadata), returns the name of the data file.
         #
@@ -380,7 +432,7 @@ class DwCAReader(object):
         """Load the archive (scientific) Metadata file, parse it with\
         ElementTree and return its content (or `None` if the archive has no metadata).
 
-        :raises: :class:`dwca.exceptions.InvalidArchive` if the archive references an non-existent
+        :raises: :class:`dwca.exceptions.InvalidArchive` if the archive references a non-existent
         metadata file.
         """
         # If the archive has descriptor, look for the metadata filename there.
@@ -391,21 +443,33 @@ class DwCAReader(object):
                 return self._parse_xml_included_file(filename)
             except IOError as exc:
                 if exc.errno == ENOENT:  # File not found
-                    msg = "{} is referenced in the archive descriptor but missing.".format(filename)
+                    msg = "{} is referenced in the archive descriptor but missing.".format(
+                        filename
+                    )
                     raise InvalidArchive(msg)
 
         else:  # Otherwise, the metadata file has to be named 'EML.xml'
             try:
                 return self._parse_xml_included_file(self.default_metadata_filename)
             except IOError as exc:
-                if exc.errno == ENOENT:  # File not found, this is an archive without metadata
+                if (
+                    exc.errno == ENOENT
+                ):  # File not found, this is an archive without metadata
                     return None
 
         assert False  # For MyPy, see: https://github.com/python/mypy/issues/4223#issuecomment-342865133
 
     def _parse_xml_included_file(self, relative_path: str) -> Element:
         """Load, parse and returns (as ElementTree.Element) XML file located at relative_path."""
-        return ET.parse(self.absolute_temporary_path(relative_path)).getroot()
+        try:
+            return ET.parse(self.absolute_temporary_path(relative_path)).getroot()
+        except ET.ParseError:
+            # In case of parsing error, it might be because of https://github.com/gbif/portal-feedback/issues/4533
+            # In that case, we work with a stripped string instead.
+            # Note that this method cannot be generalized because it won't work well with encoding specified in the xml
+            # tag. This is why we're only choosing it in case of error
+            data_as_string = self.open_included_file(relative_path, "r").read()
+            return ET.fromstring(data_as_string.strip())
 
     def _unzip_or_untar(self) -> str:
         """Create a temporary dir. and uncompress/unarchive self.archive_path there.
@@ -420,13 +484,16 @@ class DwCAReader(object):
         try:
             # Security note: with Python < 2.7.4, a zip file may be able to write outside of the
             # directory using absolute paths, parent (..) path, ... See note in ZipFile.extract doc
-            zipfile.ZipFile(self.archive_path, 'r').extractall(tmp_dir)
+            zipfile.ZipFile(self.archive_path, "r").extractall(tmp_dir)
         except zipfile.BadZipfile:
             # Doesn't look like a valid zip, let's see if it's a tar archive (possibly compressed)
             try:
-                tarfile.open(self.archive_path, 'r:*').extractall(tmp_dir)
+                # TODO: Once we only support Python 3.12+, we should pass the filter="data" argument to extractall()
+                tarfile.open(self.archive_path, "r:*").extractall(tmp_dir)
             except tarfile.ReadError:
-                raise InvalidArchive("The archive cannot be read. Is it a .zip or .tgz file?")
+                raise InvalidArchive(
+                    "The archive cannot be read. Is it a .zip or .tgz file?"
+                )
 
         return tmp_dir
 
@@ -460,15 +527,17 @@ class DwCAReader(object):
         self.core_file.close()
         for extension_file in self.extension_files:
             extension_file.close()
+        if self._metafile_handle:
+            self._metafile_handle.close()
 
         if self._directory_to_clean:
-            rmtree(self._directory_to_clean, False)
+            remove_tree(self._directory_to_clean)
 
     def core_contains_term(self, term_url: str) -> bool:
         """Return `True` if the Core file of the archive contains the `term_url` term."""
         return term_url in self.core_file.file_descriptor.terms
 
-    def __iter__(self) -> 'DwCAReader':
+    def __iter__(self) -> "DwCAReader":
         self._corefile_pointer = 0
         return self
 
